import { webpack, WebpackOptionsValidationError } from './plugins';
import { createDomain, PromiseDefer } from './util';
import Server from 'webpack-dev-server';
import createLogger from 'webpack-dev-server/lib/utils/createLogger';

function colorInfo(msg) {
  return "\x1B[1m\x1B[34m" + msg + "\x1B[39m\x1B[22m";
}

function colorError(msg) {
  return "\x1B[1m\x1B[31m" + msg + "\x1B[39m\x1B[22m";
}

process.on('unhandledRejection', function (err) {
  throw err;
});
export function startDevServer(context) {
  var webpackConfig = context.webpackConfig;
  var firstWpOpt = Array.isArray(webpackConfig) ? webpackConfig[0] : webpackConfig;
  var options = firstWpOpt.devServer || {};
  var log = createLogger(options);
  Server.addDevServerEntrypoints(webpackConfig, options);
  var defer = PromiseDefer();
  var compiler;

  try {
    compiler = webpack(webpackConfig);
  } catch (e) {
    if (e instanceof WebpackOptionsValidationError) {
      log.error(colorError(e.message));
      process.exit(1);
    }

    defer.reject(e);
  }

  var server;

  try {
    server = new Server(compiler, options, log);
  } catch (e) {
    log.error(colorError(e.message));
    process.exit(1);
    defer.reject(e);
  }

  ;
  ['SIGINT', 'SIGTERM'].forEach(function (sig) {
    process.on(sig, function () {
      server.close();
      process.exit();
    });
  });
  server.listen(options.port, options.host, function (err) {
    if (err) throw err;
    defer.resolve(server);
    log.info("\nService is running at " + colorInfo(createDomain(options)));
  });
  return defer.promise;
}