import * as tslib_1 from "tslib";
import CaseSensitivePathsPlugin from 'case-sensitive-paths-webpack-plugin';
import OptimizeCSSAssetsPlugin from 'optimize-css-assets-webpack-plugin';
import { BundleAnalyzerPlugin } from 'webpack-bundle-analyzer';
import LodashWebpackPlugin from 'lodash-webpack-plugin';
import TerserPlugin from 'terser-webpack-plugin';
import path from 'path';
import { ProgressPlugin, IgnorePlugin, AggressiveSplittingPlugin } from './plugins';
import { getProgressHandler, getOptions } from './util';
export default function mwPlugins(context, next) {
  return tslib_1.__awaiter(this, void 0, void 0, function () {
    var compress, plugins, dll, ENV, _a, aggressive, __IS_BUILD_LIBRARY__, cwd, outputPath, _b, records, _c, analyzer, _d, optimizeLodash;

    return tslib_1.__generator(this, function (_e) {
      context.plugins = [new CaseSensitivePathsPlugin(), new ProgressPlugin(getProgressHandler), new IgnorePlugin(/^\.\/locale$/, /moment$/)];
      next();
      compress = context.compress, plugins = context.plugins, dll = context.dll, ENV = context.ENV, _a = context.aggressive, aggressive = _a === void 0 ? false : _a, __IS_BUILD_LIBRARY__ = context.__IS_BUILD_LIBRARY__;
      context.webpackConfig.optimization = context.webpackConfig.optimization || {};

      if (!Array.isArray(dll)) {
        context.webpackConfig.optimization = {
          splitChunks: {
            chunks: 'all',
            name: false
          },
          runtimeChunk: !__IS_BUILD_LIBRARY__,
          removeAvailableModules: true,
          removeEmptyChunks: true,
          mergeDuplicateChunks: true,
          sideEffects: false,
          flagIncludedChunks: true,
          occurrenceOrder: true,
          concatenateModules: true,
          usedExports: true,
          providedExports: true,
          noEmitOnErrors: true,
          namedModules: ENV.isDevelopment,
          namedChunks: ENV.isDevelopment
        };

        if (aggressive) {
          plugins.push(new AggressiveSplittingPlugin(getOptions(aggressive)));
        }
      }

      context.webpackConfig.optimization.minimize = !!compress;

      if (compress) {
        context.webpackConfig.optimization.minimizer = [new TerserPlugin({
          terserOptions: {
            parse: {
              ecma: 8
            },
            compress: {
              ecma: 5,
              warnings: false,
              comparisons: false,
              inline: 2
            },
            mangle: {
              safari10: true
            },
            output: {
              ecma: 5,
              comments: false,
              ascii_only: true
            }
          },
          parallel: true,
          cache: true,
          sourceMap: false
        }), new OptimizeCSSAssetsPlugin({
          cssProcessorOptions: {
            map: false
          }
        })];
      }

      cwd = context.cwd, outputPath = context.outputPath, _b = context.records, records = _b === void 0 ? false : _b, _c = context.analyzer, analyzer = _c === void 0 ? false : _c, _d = context.optimizeLodash, optimizeLodash = _d === void 0 ? true : _d;
      context.plugins = plugins;

      if (records) {
        context.webpackConfig.recordsOutputPath = path.join(cwd, outputPath, getOptions(records, 'records.json'));
      }

      if (analyzer) {
        plugins.push(new BundleAnalyzerPlugin(getOptions(analyzer)));
      }

      if (optimizeLodash) {
        plugins.push(new LodashWebpackPlugin(getOptions(optimizeLodash, {
          shorthands: true,
          paths: true,
          cloning: true,
          flattening: true,
          exotics: true,
          collections: true,
          caching: true
        })));
      }

      return [2];
    });
  });
}