import _typeof from "@babel/runtime/helpers/typeof";
import * as tslib_1 from "tslib";
import path from 'path';
import fs from 'fs';
import mwBuild from './mwBuild';
import mwBabelOptions from './mwBabelOptions';
import mwPostCSSOptions from './mwPostCSSOptions';
import mwTSOptions from './mwTSOptions';
import mwDevServer from './mwDevServer';
import mwMultiEntryHTML from './mwMultiEntryHTML';
import mwPlugins from './mwPlugins';
import mwRules from './mwRules';
import mwENV from './mwENV';
import mwDLL from './mwDLL';
import mwPWA from './mwPWA';
import merge from 'webpack-merge';

function noop() {
  return tslib_1.__awaiter(this, void 0, void 0, function () {
    return tslib_1.__generator(this, function (_a) {
      return [2];
    });
  });
}

function getCustomConfig(cwd, config) {
  var _this = this;

  if ('function' === typeof config) {
    return config;
  }

  var paths = [];

  switch (process.env.NODE_ENV) {
    case 'production':
      paths.push.apply(paths, ['build', 'production', 'prod']);
      break;

    case 'development':
      paths.push.apply(paths, ['development', 'develop', 'dev']);
      break;

    case 'beta':
      paths.push.apply(paths, ['beta', 'build', 'production', 'prod', 'development', 'develop', 'dev']);
      break;

    default:
      paths.push(process.env.NODE_ENV);
      break;
  }

  paths.push('config');
  paths = paths.map(function (name) {
    return "webpack." + name + ".js";
  });
  config && paths.unshift(config);
  var cc = noop;
  var i = -1;

  while (++i < paths.length) {
    var p = paths[i];
    var pp = path.join(cwd, p);

    if (fs.existsSync(pp)) {
      console.info("" + pp);
      cc = require(pp);
      break;
    }
  }

  if ('function' === typeof cc) {
    return cc;
  }

  if ('object' === _typeof(cc)) {
    return function (ctx) {
      return tslib_1.__awaiter(_this, void 0, void 0, function () {
        return tslib_1.__generator(this, function (_a) {
          ctx.webpackConfig = merge(ctx.webpackConfig, cc);

          if (cc.target && cc.output.libraryTarget) {
            ctx.__IS_BUILD_LIBRARY__ = true;
          }

          return [2];
        });
      });
    };
  }

  return null;
}

export function mwsBuild(cwd, config) {
  var mwConfig = getCustomConfig(cwd, config);
  return [mwENV, mwBuild, mwPWA, mwMultiEntryHTML, mwDLL, mwPlugins, mwRules, mwBabelOptions, mwPostCSSOptions, mwTSOptions, mwConfig];
}
export function mwsDevServer(cwd, config) {
  var mwConfig = getCustomConfig(cwd, config);
  return [mwENV, mwBuild, mwPWA, mwMultiEntryHTML, mwDLL, mwPlugins, mwRules, mwBabelOptions, mwPostCSSOptions, mwTSOptions, mwDevServer, mwConfig];
}
export function mwsDLL(cwd, config) {
  var mwConfig = getCustomConfig(cwd, config);
  return [mwENV, mwBuild, mwDLL, mwPlugins, mwRules, mwBabelOptions, mwPostCSSOptions, mwTSOptions, mwConfig];
}