"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = mwPlugins;

var tslib_1 = _interopRequireWildcard(require("tslib"));

var _caseSensitivePathsWebpackPlugin = _interopRequireDefault(require("case-sensitive-paths-webpack-plugin"));

var _optimizeCssAssetsWebpackPlugin = _interopRequireDefault(require("optimize-css-assets-webpack-plugin"));

var _webpackBundleAnalyzer = require("webpack-bundle-analyzer");

var _lodashWebpackPlugin = _interopRequireDefault(require("lodash-webpack-plugin"));

var _terserWebpackPlugin = _interopRequireDefault(require("terser-webpack-plugin"));

var _path = _interopRequireDefault(require("path"));

var _plugins = require("./plugins");

var _util = require("./util");

function mwPlugins(context, next) {
  return tslib_1.__awaiter(this, void 0, void 0, function () {
    var compress, plugins, dll, ENV, _a, aggressive, __IS_BUILD_LIBRARY__, cwd, outputPath, _b, records, _c, analyzer, _d, optimizeLodash;

    return tslib_1.__generator(this, function (_e) {
      context.plugins = [new _caseSensitivePathsWebpackPlugin.default(), new _plugins.ProgressPlugin(_util.getProgressHandler), new _plugins.IgnorePlugin(/^\.\/locale$/, /moment$/)];
      next();
      compress = context.compress, plugins = context.plugins, dll = context.dll, ENV = context.ENV, _a = context.aggressive, aggressive = _a === void 0 ? false : _a, __IS_BUILD_LIBRARY__ = context.__IS_BUILD_LIBRARY__;
      context.webpackConfig.optimization = context.webpackConfig.optimization || {};

      if (!Array.isArray(dll)) {
        context.webpackConfig.optimization = {
          splitChunks: {
            chunks: 'all',
            name: false
          },
          runtimeChunk: !__IS_BUILD_LIBRARY__,
          removeAvailableModules: true,
          removeEmptyChunks: true,
          mergeDuplicateChunks: true,
          sideEffects: false,
          flagIncludedChunks: true,
          occurrenceOrder: true,
          concatenateModules: true,
          usedExports: true,
          providedExports: true,
          noEmitOnErrors: true,
          namedModules: ENV.isDevelopment,
          namedChunks: ENV.isDevelopment
        };

        if (aggressive) {
          plugins.push(new _plugins.AggressiveSplittingPlugin((0, _util.getOptions)(aggressive)));
        }
      }

      context.webpackConfig.optimization.minimize = !!compress;

      if (compress) {
        context.webpackConfig.optimization.minimizer = [new _terserWebpackPlugin.default({
          terserOptions: {
            parse: {
              ecma: 8
            },
            compress: {
              ecma: 5,
              warnings: false,
              comparisons: false,
              inline: 2
            },
            mangle: {
              safari10: true
            },
            output: {
              ecma: 5,
              comments: false,
              ascii_only: true
            }
          },
          parallel: true,
          cache: true,
          sourceMap: false
        }), new _optimizeCssAssetsWebpackPlugin.default({
          cssProcessorOptions: {
            map: false
          }
        })];
      }

      cwd = context.cwd, outputPath = context.outputPath, _b = context.records, records = _b === void 0 ? false : _b, _c = context.analyzer, analyzer = _c === void 0 ? false : _c, _d = context.optimizeLodash, optimizeLodash = _d === void 0 ? true : _d;
      context.plugins = plugins;

      if (records) {
        context.webpackConfig.recordsOutputPath = _path.default.join(cwd, outputPath, (0, _util.getOptions)(records, 'records.json'));
      }

      if (analyzer) {
        plugins.push(new _webpackBundleAnalyzer.BundleAnalyzerPlugin((0, _util.getOptions)(analyzer)));
      }

      if (optimizeLodash) {
        plugins.push(new _lodashWebpackPlugin.default((0, _util.getOptions)(optimizeLodash, {
          shorthands: true,
          paths: true,
          cloning: true,
          flattening: true,
          exotics: true,
          collections: true,
          caching: true
        })));
      }

      return [2];
    });
  });
}