"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getProgressHandler = getProgressHandler;
exports.checkWebpackConfig = checkWebpackConfig;
exports.getValueByPath = getValueByPath;
exports.getBuildCopyFiles = getBuildCopyFiles;
exports.getBuildExternals = getBuildExternals;
exports.getBuild4DevelopmentENV = getBuild4DevelopmentENV;
exports.getBuild4ProductionENV = getBuild4ProductionENV;
exports.getBuildHTMLData = getBuildHTMLData;
exports.getBuildHTML = getBuildHTML;
exports.createDomain = createDomain;
exports.getNodeVersion = getNodeVersion;
exports.getBuildSourceMap = getBuildSourceMap;
exports.getBuildTemplateParametersWithScripts = getBuildTemplateParametersWithScripts;
exports.getOptions = getOptions;
exports.PromiseDefer = PromiseDefer;
exports.getExampleEntry = getExampleEntry;
Object.defineProperty(exports, "notifier", {
  enumerable: true,
  get: function get() {
    return _nodeNotifier.default;
  }
});
Object.defineProperty(exports, "chalk", {
  enumerable: true,
  get: function get() {
    return _chalk.default;
  }
});

var tslib_1 = _interopRequireWildcard(require("tslib"));

var _nodeNotifier = _interopRequireDefault(require("node-notifier"));

var _chalk = _interopRequireDefault(require("chalk"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

function getProgressHandler(percent, msg1, msg2) {
  var stream = process.stdout;

  if (stream.isTTY && percent < 0.7) {
    stream.cursorTo(0);
    stream.write("\u231B  " + _chalk.default.magenta(msg2) + " " + msg1);
    stream.clearLine(1);
  } else if (percent >= 1) {
    console.log(_chalk.default.green('\nwebpack: bundle build is now finished'));
  }
}

function checkWebpackConfig(webpackConfig) {
  var configs = Array.isArray(webpackConfig) ? webpackConfig : [webpackConfig];
  var hasEmptyEntry = configs.some(function (c) {
    return Object.keys(c.entry || {}).length === 0;
  });

  if (hasEmptyEntry) {
    var e = new Error('no webpack entry found');
    e.name = 'NoEntry';
    throw e;
  }
}

function getValueByPath(path) {
  return !_fs.default.existsSync(path) ? {} : require(path);
}

function getBuildCopyFiles(files) {
  if (Array.isArray(files)) {
    return files.map(function (from) {
      return {
        from: from
      };
    });
  }

  return Object.keys(files).filter(function (key) {
    var file = files[key];
    return typeof file === 'string' || !!file.path;
  }).map(function (key) {
    var file = files[key];

    if (typeof file === 'string') {
      return {
        from: file
      };
    }

    return {
      from: file.path,
      to: file.to
    };
  });
}

function getBuildExternals(files) {
  var ret = {};
  Object.keys(files).forEach(function (key) {
    var file = files[key];

    if (typeof file === 'string') {
      ret[key] = file;
    } else if (file.name) {
      ret[key] = file.name;
    }
  });
  return ret;
}

function getBuild4DevelopmentENV(filesMap) {
  var ret = [];
  Object.keys(filesMap).forEach(function (key) {
    var line = filesMap[key];
    var path = line.path;

    if (line.name && path) {
      ret.push(path);
    }
  });
  return ret;
}

function getBuild4ProductionENV(filesMap) {
  var ret = [];
  Object.keys(filesMap).forEach(function (key) {
    var line = filesMap[key];
    var path = line.path;

    if (line.name && path) {
      var paths = path.split('/');
      path = paths[paths.length - 1];
      ret.push(path);
    }
  });
  return ret;
}

function getBuildHTMLData(filesMap, env) {
  switch (env) {
    case 'production':
    case 'beta':
      return getBuild4ProductionENV(filesMap);

    default:
      return getBuild4DevelopmentENV(filesMap);
  }
}

function getBuildHTML(context) {
  var _a = context.externals,
      externals = _a === void 0 ? {} : _a,
      _b = context.sdks,
      sdks = _b === void 0 ? {} : _b,
      DLL_FILENAME = context.DLL_FILENAME,
      ENV = context.ENV,
      htmlWebpackPluginOptions = context.htmlWebpackPluginOptions,
      compress = context.compress;
  var entry = context.entry || context.packageMap.entry || getExampleEntry(context);

  if (!entry) {
    throw new Error('entry is an invalid value');
  }

  if (typeof entry === 'string') {
    context.entry = entry = {
      index: entry
    };
  }

  var paths = getBuildHTMLData(externals, ENV.env);
  var entryNames = Object.keys(entry);

  var options = tslib_1.__assign({
    chunksSortMode: 'none',
    template: _path.default.join(__dirname, '../assets/index.hbs'),
    favicon: _path.default.join(__dirname, '../assets/favicon.ico')
  }, htmlWebpackPluginOptions);

  return entryNames.map(function (entryName) {
    var sdk = sdks[entryName];

    if (typeof sdk === 'string') {
      sdk = [sdk];
    }

    var scripts = paths.slice(0);

    if (sdk) {
      scripts.push.apply(scripts, sdk);
    }

    if (DLL_FILENAME) {
      scripts.push(DLL_FILENAME);
    }

    return tslib_1.__assign({
      PATHS: scripts,
      scripts: scripts,
      entryName: entryName,
      filename: entryName + ".html",
      chunks: [entryName],
      chunksSortMode: 'dependency',
      inject: true,
      templateParameters: getBuildTemplateParametersWithScripts(scripts),
      minify: compress ? {
        removeComments: true,
        collapseWhitespace: true,
        removeRedundantAttributes: true,
        useShortDoctype: true,
        removeEmptyAttributes: true,
        removeStyleLinkTypeAttributes: true,
        keepClosingSlash: true,
        minifyJS: true,
        minifyCSS: true,
        minifyURLs: true
      } : null
    }, options);
  });
}

function createDomain(_a) {
  var host = _a.host,
      port = _a.port;
  return "http://" + host + ":" + port;
}

function getNodeVersion(packageMap) {
  var emptyBuildIns = ['child_process', 'cluster', 'dgram', 'dns', 'fs', 'module', 'net', 'readline', 'repl', 'tls'];
  var browser = packageMap.browser || {};
  return emptyBuildIns.reduce(function (obj, name) {
    var _a;

    if (!(name in browser)) {
      return tslib_1.__assign({}, obj, (_a = {}, _a[name] = 'empty', _a));
    }

    return obj;
  }, {});
}

function getBuildSourceMap(devtool, ENV) {
  if (devtool === void 0) {
    devtool = false;
  }

  if (devtool === true) {
    devtool = ENV.isProduction ? false : '#cheap-module-eval-source-map';
  }

  return devtool;
}

function getBuildTemplateParametersWithScripts(scripts) {
  return function (compilation, assets, options) {
    var entryName = options.entryName;
    var stats = compilation.getStats().toJson();
    var currentAssets = stats.entrypoints[entryName].assets;
    var js = currentAssets.filter(function (n) {
      return _path.default.extname(n) === '.js';
    });
    var css = currentAssets.filter(function (n) {
      return _path.default.extname(n) === '.css';
    });
    assets.js = unique(scripts.concat(assets.js).concat(js));
    assets.css = unique(assets.css.concat(css));
    return {
      compilation: compilation,
      webpack: compilation.getStats().toJson(),
      webpackConfig: compilation.options,
      htmlWebpackPlugin: {
        files: assets,
        options: options,
        scripts: js,
        styles: css
      }
    };
  };
}

function unique(array) {
  return array.filter(function (item, index, array) {
    return array.indexOf(item) === index;
  });
}

function getOptions(options, defaultOptions) {
  if (defaultOptions === void 0) {
    defaultOptions = {};
  }

  return options === true ? defaultOptions : options;
}

function PromiseDefer() {
  var resolve = void 0;
  var reject = void 0;
  var promise = new Promise(function (rs, rj) {
    resolve = rs;
    reject = rj;
  });
  return {
    promise: promise,
    resolve: resolve,
    reject: reject
  };
}

function getExampleEntry(context) {
  if (!context.ENV.isDevelopment) {
    return null;
  }

  var entry = null;
  ['js', 'jsx', 'ts', 'tsx'].forEach(function (ext) {
    if (_fs.default.existsSync(_path.default.join(context.DIRs.root, 'example', "index." + ext))) {
      entry = "./example/index." + ext;
    }
  });
  return entry;
}